@@ -116,7 +116,8 @@ get_window_for_shape (ThemeImage *image, GdkWindow *window, GtkWidget *widget,
    * Noticed when GtkMenu was changed to do two paints, the other one being on
    * ->bin_window (http://bugzilla.gnome.org/show_bug.cgi?id=169532)
    */
-  if (image->background_shaped && window == widget->window)
+  if (image->background_shaped &&
+      widget && window == widget->window)
     {
       gint window_width;
       gint window_height;
@@ -246,7 +247,7 @@ draw_simple_image(GtkStyle       *style,
     }
 
   /* Check for maemo-position-theming to update the position data */
-  if (widget->parent &&
+  if (widget && widget->parent &&
       gtk_widget_class_find_style_property (GTK_WIDGET_GET_CLASS (widget->parent),
                                             "maemo-position-theming"))
     {
@@ -676,6 +677,10 @@ draw_arrow (GtkStyle     *style,
 				box_x, box_y, box_width, box_height);
     }
 
+#if GTK_CHECK_VERSION(2,9,2)
+  if (arrow_direction == GTK_ARROW_NONE)
+    return;
+#endif
 
   match_data.function = TOKEN_D_ARROW;
   match_data.detail = (gchar *)detail;
@@ -782,6 +787,14 @@ maybe_check_submenu_state (GtkMenuItem *menu_item, ThemeMatchData *match_data)
       if (sub_item && GTK_WIDGET_STATE (sub_item) != GTK_STATE_NORMAL)
 	match_data->state = GTK_STATE_SELECTED;
     }
+
+  /* This is a hack to cover the cases where a menu item is being 
+   * used outside the usual a GtkMenu. More specificaly, this covers
+   * menu items inside the hildon-desktop custom menu widget. */
+  if (!GTK_IS_MENU (GTK_WIDGET (menu_item)->parent) &&
+      !gtk_widget_is_focus (GTK_WIDGET (menu_item)) &&
+      GTK_WIDGET_STATE (menu_item) != GTK_STATE_NORMAL)
+    match_data->state = GTK_STATE_SELECTED;
 }
 
 static void
@@ -1090,7 +1103,7 @@ draw_expander (GtkStyle        *style,
                GtkExpanderStyle expander_style)
 {
   ThemeMatchData match_data;
-  gint expander_size;
+  gint expander_size = 10;
 
   g_return_if_fail(style != NULL);
   g_return_if_fail(window != NULL);
@@ -1100,7 +1113,8 @@ draw_expander (GtkStyle        *style,
    * XXX Might want to introduce proper keywords for expanders some day.
    */
 
-  gtk_widget_style_get (widget, "expander-size", &expander_size, NULL);
+  if (widget)
+    gtk_widget_style_get (widget, "expander-size", &expander_size, NULL);
 
   match_data.function = TOKEN_D_ARROW;
   match_data.detail = (gchar *)detail;
